import scipy.io.wavfile as wavfile
import scipy
import scipy.fftpack
import wave
import numpy as np
import struct
from matplotlib import pyplot as plt
from scipy.signal import hilbert


file = "Echantillon_Violon_haute_qualité_2.wav"
fs_rate, signal = wavfile.read(file)
print ("Fréquence d'échantillonage : ", fs_rate)

# Nbre de voies
l_audio = len(signal.shape)
print ("nbre de voies :  ", l_audio)
if l_audio == 2:
    signal = signal.sum(axis=1) / 2

# caractéristiques du signal
N = signal.shape[0]
print ("Nbre d'échantillons : ", N)
N2 = int(N/2)
secs = N / float(fs_rate)
print ("durée de l'échantillon (s) : ", secs)
Ts = 1.0/fs_rate # Temps d'échantillonage
print ("temps d'échantillonage : ", Ts)

### Transformée de Fourier du signal
t = scipy.arange(0, secs, Ts)
#Génération des fréquences
freqs = np.fft.fftfreq(signal.size, Ts)
#calcul de la composante de la série de Fourier sous forme complexe
# sinx = 1/(2j)*(e(jx) - e(-jx))
# cosx = 1/2*(e(jx) + e(-jx))
sp = np.fft.fft(signal)

#Détermination du module pour chaque composante de la série de Fourier
Amplitude = np.abs(sp)/signal.size
Amplitude[np.abs(sp) < 1] = 0  #Amplitude = 0 pour les amplitudes négligeables

#Détermination de la phase pour chaque composante de la série de Fourier
phase = np.angle(sp)
phase[np.abs(sp) < 1] = 0 #phase = 0 pour les amplitudes négligeables

# Ne prendre que la moitié du spectre
Amplitude_side = 2*Amplitude[range(N2)]# on ne représente que la partie positive des fréquences
Freqs_side = freqs[range(N2)]
Phase_side = phase[range(N2)]
t_side = t[range(N2)]

# Sélection seulement des éléments significatifs dans le spectre
# Et simplification du spectre
Amplitude_Strong = Amplitude_side[Amplitude_side > np.max(Amplitude_side)/ 10]

Freqs_Strong = Freqs_side[Amplitude_side > np.max(Amplitude_side)/ 10]
Phase_Strong = Phase_side[Amplitude_side > np.max(Amplitude_side)/ 10]
Freqs_Norm = Freqs_side/Freqs_Strong[0]*440

print("fondamental : ",Freqs_Strong[0]," Hz")

def append_silence(duration_milliseconds=100):
    num_samples = duration_milliseconds * (sample_rate / 1000.0)
    for x in range(int(num_samples)): 
        audio.append(0.0)
    return


def append_note(
        freq=440.0, 
        duration_milliseconds=500, 
        volume=1.0):
    global audio 
    num_samples = duration_milliseconds * (sample_rate / 1000.0)
    envelope1 = scipy.arange(0.00,1.00, 1/int(num_samples/2))
    envelope2 = sorted(envelope1, reverse = True)
    envelope = np.concatenate((envelope1 , envelope2))
    for x in range(int(num_samples)):
        y = 0.00
        for j in range (len(Freqs_side)) :
            y += Amplitude_side[j]*envelope[x]*np.cos(2*np.pi*Freqs_side[j]/Freqs_Strong[0]*freq*(x/sample_rate)+ Phase_side[j])
        audio.append(volume * y)
    return


def save_wav(file_name):
    wav_file=wave.open(file_name,"w")
    nchannels = 1
    sampwidth = 2
    # 44100 Hz est le tout d'échantillongae pour CD audio.
    # Si vous avez besoin d'une plus basse qualité vous pouvez ajuster au-dessus sample_rate.
    # La qualité standard basse est de 8 kHz
    nframes = len(audio)
    comptype = "NONE"
    compname = "not compressed"
    wav_file.setparams((nchannels, sampwidth, sample_rate, nframes, comptype, compname))

    for sample in audio:
        wav_file.writeframes(struct.pack('h', int( sample * 32767.0 )))

    wav_file.close()

    return

### Compléter le script indiquant la bonne fréquence et la bonne durée 
append_note(freq = ..., duration_milliseconds = ....., volume=1)

### Exportation du fichier wav
### Si la bibliothèque est disponible on peut utiliser sounddevice
save_wav("Au_Clair_de_lune_violon.wav")
print("Création du fichier wav terminé")

#Tracé du spectre de Fourier
figure = plt.figure(figsize = (8, 6))
plt.gcf().subplots_adjust(left = 0.1, bottom = 0.1, right = 0.95, top = 0.95, wspace = 0, hspace = 0.5)
axes = figure.add_subplot(3, 1, 1)
plt.plot(t, signal, "g")
axes.set_xlabel('temps (s)')
axes.set_ylabel('Amplitude')
axes.set_title('Signal enregistré')

axes = figure.add_subplot(3, 1, 2)
plt.plot(Freqs_side, Amplitude_side, "b")
axes.set_xlabel('Frequency (Hz)')
axes.set_ylabel('Amplitude')
axes.set_xlim(0,5000)
axes.set_title('Spectre de Fourier ')

axes = figure.add_subplot(3, 1, 3)
plt.plot(Freqs_side, Phase_side, "r")
axes.set_xlabel('Frequency (Hz)')
axes.set_ylabel('Phase (rad)')
axes.set_xlim(0,5000)
axes.set_title('Spectre de phase ')

plt.show()



